@@ -1,14 +1,14 @@
 # ---------------------------------------------------------------
-#                   Figure Refinery 
+#                   Figure Refinery
 # ---------------------------------------------------------------
 
-#  Last modified  5 November 2008
+# Last modified 2008.12.26 by Spencer Graves
+# Previously modified  5 November 2008
 
 #  attach the FDA functions
 
-#  input the data
-
-refinery <- t(matrix(scan("../data/refinery.txt", 0), 3, 194))
+# 'refinery' is a data set in 'fda' so no need to read it.
+#refinery <- t(matrix(scan("../data/refinery.txt", 0), 3, 194))
 
 tval <- refinery[,1]  #  observation time
 uval <- refinery[,2]  #  reflux flow
@@ -22,16 +22,17 @@ yval <- yval - mean(yval[1:60])
 
 #  plot data
 
-par(mfrow=c(2,1), pty="m")
-plot(tval, yval, type="p", ylim=c(-0.5,4.5), 
+op <- par(mfrow=c(2,1), pty="m")
+plot(tval, yval, type="p", ylim=c(-0.5,4.5),
      xlab="Time", ylab="Tray 47 level")
 
-plot(tval, uval, type="p", ylim=c(-0.6,0.2), 
+plot(tval, uval, type="p", ylim=c(-0.6,0.2),
      xlab="Time", ylab="Reflux flow")
+par(op)
 
 #  ------------------------------------------------------------
 #    A forced first order constant coefficient DIFE
-#        exp(-wt)[x.0 + D^{-1} [exp(wt) u(t)] 
+#        exp(-wt)[x.0 + D^{-1} [exp(wt) u(t)]
 #    Annihilating operator is L = w x + Dx
 #    The data are the Corpus Christi refinery data
 #  ------------------------------------------------------------
@@ -53,7 +54,8 @@ ufd <- smooth.basis(tval, uval, ubasis)$fd
 #  put three coincident knots at tbreak
 
 norder  <-  4
-yknots  <- c(seq(0, tbreak, len=3), tbreak, seq(tbreak, refrange[2], len=5))
+yknots  <- c(seq(0, tbreak, len=3), tbreak,
+             seq(tbreak, refrange[2], len=5))
 nybasis <- length(yknots) + norder - 2
 ybasis  <- create.bspline.basis(refrange, nybasis, norder, yknots)
 
@@ -64,20 +66,22 @@ yvec <- eval.fd(tval, yfd)
 #  plot the data with fits and knots
 
 par(mfrow=c(1,1))
-plot(tval, yval, type="p", xlab="minutes", ylab="Tray 47 level", xlim=refrange)
+plot(tval, yval, type="p", xlab="minutes",
+     ylab="Tray 47 level", xlim=refrange)
 lines(tval, yvec, lwd=3)
-for (i in 2:length(yknots)-1) 
+for (i in 2:length(yknots)-1)
     lines(c(yknots[i],yknots[i]), c(0,4), lty=4, lwd=2)
 
 
-#  set up fRegress concurrent model analysis, 
+#  set up fRegress concurrent model analysis,
 #  with the dependent variable being tray 47 level
 #  and independent variable being reflux flow
 
 p <- 1
 
 norder    <- 4
-bbreaks   <- c(seq(0, tbreak, len=2), tbreak, seq(tbreak, refrange[2], len=5))
+bbreaks   <- c(seq(0, tbreak, len=2), tbreak,
+               seq(tbreak, refrange[2], len=5))
 nbbasis   <- length(bbreaks) + norder - 2
 bbasis    <- create.bspline.basis(refrange, nbbasis, norder, bbreaks)
 betafd    <- fd(matrix(0,nbbasis,p), bbasis)
@@ -91,7 +95,7 @@ xfdlist  <- list(ufd)
 
 fRegressList <- fRegress(yfd, xfdlist, betalist)
 
-betaestlist <- fRegressList$betaestlist 
+betaestlist <- fRegressList$betaestlist
 yhatfdobj   <- fRegressList$yhatfdobj
 
 #  plot the regression function
@@ -101,7 +105,7 @@ betafd    <- betafdPar$fd
 
 plot(betafd, xlab="minutes", ylab="beta(t)")
 
-#  set up fRegress concurrent model analysis, 
+#  set up fRegress concurrent model analysis,
 #  with the dependent variable being derivative of tray 47 level
 #  and independent variables being tray 47 level and reflux flow
 
@@ -129,12 +133,12 @@ xfdlist[[2]] <- ufd
 
 fRegressList <- fRegress(Dyfd, xfdlist, betalist)
 
-betaestlist <- fRegressList$betaestlist 
+betaestlist <- fRegressList$betaestlist
 Dyhatfdobj  <- fRegressList$yhatfdobj
 
 betafdPar1 <- betaestlist[[1]]
 betafdPar2 <- betaestlist[[2]]
-beta1 <- betafdPar1$fd$coefs  
+beta1 <- betafdPar1$fd$coefs
 beta2 <- betafdPar2$fd$coefs
 
 print(paste("Beta_1 =",round(beta1,3)))
@@ -142,7 +146,8 @@ print(paste("Beta_2 =",round(beta2,3)))
 
 #  plot the first derivative and fit
 
-Dyhatvec <- eval.fd(tvalD, Dyhatfdobj)
+#Dyhatvec <- eval.fd(tvalD, Dyhatfdobj)
+Dyhatvec <- predict(Dyhatfdobj, tvalD)
 
 plot(Dyfd, ylim=c(-.02,0.1))
 lines(tvalD, Dyhatvec, lwd=3)
@@ -156,7 +161,7 @@ yhatvec[68:nD] <- 0.4924*(beta2/beta1)*(1-exp(beta1*(tval[68:nD]-67)))
 #  plot the data and fit
 
 par(mfrow=c(1,1))
-plot(tval, yval, type="p", xlim=refrange, 
+plot(tval, yval, type="p", xlim=refrange,
      xlab="minutes", ylab="Tray 47 level")
 lines(tvalD, yhatvec, lwd=3)
 
